local *

{
  :GameTimeFrame
  :MiniMapWorldMapButton
  :Minimap
  :MinimapBorder
  :MinimapBorderTop
  :MinimapCluster
  :MinimapNorthTag
  :MinimapToggleButton
  :MinimapZoneText
  :MinimapZoneTextButton
  :MinimapZoomIn
  :MinimapZoomOut
  :UIParent
  :idTweaks
  :idTweaksActionBar1
  :idTweaksExperienceBar
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :TL, :TC, :TR,
  :ML, :MC, :MR,
  :BL, :BC, :BR,
} = idTweaks.anchor
{
  :Option
  :ToggleOption
} = idTweaksOptions

class MiniMap extends idTweaksModule
  new: (addon) =>
    super addon, 'MiniMap', 'AceConsole-3.0', 'AceEvent-3.0'

  set_options: =>
    super!

    @options\add_option Option @, 'shape', 'Shape', 'Shape settings', {},
      square: ToggleOption @, 'square', 'Square', 'Makes the minimap square',
        check: -> @shape_square_check!

    @options\add_option Option @, 'mouse', 'Mouse', 'Change how the mouse interacts with the MiniMap.', {},
      mouse_wheel_zooming: ToggleOption @, 'mouse_wheel_zooming', 'Mouse Wheel Zooming', 'Enables Zooming the minimap with the Mouse Wheel.',
        check: -> @mouse_mousewheel_zooming_check!

    @options\add_option Option @, 'hide', 'Hide', 'Toggle visibility of parts of the MiniMap with these options.', {},
      area_text: ToggleOption @, 'area_text', 'Area Text', 'Area Text',
        check: -> @hide_check!
      calendar_button: ToggleOption @, 'calendar_button', 'Calendar Button', 'Calendar Button',
        check: -> @hide_check!
      north_tag: ToggleOption @, 'north_tag', 'North Tag', 'North Tag',
        check: -> @hide_check!
      time_display: ToggleOption @, 'time_display', 'Time Display', 'Time Display',
        check: -> @hide_check!
      worldmap_button: ToggleOption @, 'worldmap_button', 'World Map Button', 'World Map Button',
        check: -> @hide_check!
      zoom_buttons: ToggleOption @, 'zoom_buttons', 'Zoom Buttons', 'Zoom Buttons',
        check: -> @hide_check!
    
    if idTweaks\is_bc!
      (@options\get_option 'hide')\add_option ToggleOption @, 'close_button', 'Close Button', 'Close Button',
        check: -> @hide_check!

  enable: =>
    @check!

  disable: =>
    @hide_disable!
    @shape_square_disable!
    @shape_position_disable!
    @mouse_disable!

  check: =>
    @hide_check!
    @shape_square_check!
    @shape_position_check!
    @mouse_check!

  positioning: =>
    @shape_square_check!
    @shape_position_check!

  hide_frames:
    area_text:
      enable: =>
        MinimapBorderTop\Hide!
        MinimapZoneText\Hide!
        MinimapZoneTextButton\Hide!
         -- MinimapToggleButton\Hide!

      disable: =>
        MinimapBorderTop\Show!
        MinimapZoneText\Show!
        MinimapZoneTextButton\Show!
         -- MinimapToggleButton\Show!

    calendar_button:
      enable: =>
        GameTimeFrame\Hide!

      disable: =>
        GameTimeFrame\Show!

    north_tag:
      enable: =>
        MinimapNorthTag\SetAlpha 0

      disable: =>
        MinimapNorthTag\SetAlpha 1

    time_display:
      enable: =>
        _G.TimeManagerClockButton\Hide!

      disable: =>
        _G.TimeManagerClockButton\Show!

    worldmap_button:
      enable: =>
        MiniMapWorldMapButton\Hide!

      disable: =>
        MiniMapWorldMapButton\Show!

    zoom_buttons:
      enable: =>
        MinimapZoomIn\Hide!
        MinimapZoomOut\Hide!
      disable: =>
        MinimapZoomIn\Show!
        MinimapZoomOut\Show!
        
    close_button:
      enable: =>
        return unless idTweaks\is_bc!

        MinimapToggleButton\Hide!

      disable: =>
        return unless idTweaks\is_bc!

        MinimapToggleButton\Show!

  hide_check: =>
    for name, functions in pairs @hide_frames
      @hide_check_frame name

  hide_check_frame: (name) =>
    if @db.hide[name] == true
      @hide_frames[name].enable!
    else
      @hide_frames[name].disable!

  hide_disable: =>
    for name, functions in pairs @hide_frames
      @hide_frames[name].disable!

  shape_square_check: =>
    if @db.shape.square == true
      @shape_square_enable!
    else
      @shape_square_disable!

  shape_square_enable: =>
    Minimap\SetMaskTexture 'Interface\\Buttons\\WHITE8x8'
    MinimapBorder\Hide!

  shape_square_disable: =>
    Minimap\SetMaskTexture 'textures\\MinimapMask'
    MinimapBorder\Show!

  shape_position_check: =>
     -- if idTweaks\is_module_enabled 'ActionBars'
     --   if @db.shape_position == true
     --     @shape_position_enable!
     --   else
     --     @shape_position_disable!
     -- else
     --   @shape_position_disable!

    @shape_position_disable!

  shape_position_enable: =>
    Minimap\ClearAllPoints!
    Minimap\SetParent UIParent

    import ActionBars from idTweaks.modules

    size = 0
    bar_margin = ActionBars.db.profile.bar_margin
     -- bar_padding = ActionBars.db.profile.bar_padding
     -- button_padding = ActionBars.db.profile.button_padding

    experience_bar_database = idTweaks.modules.ExperienceBar.db.profile

    if experience_bar_database.enabled
      if experience_bar_database.full_width
        _, height = ActionBars\get_total_size!
        size = height
        Minimap\SetPoint BL, idTweaksActionBar1, BR, bar_margin, bar_margin
      else
        _, height = ActionBars\getTotalSize!
        size = height + idTweaksExperienceBar\GetHeight!
        Minimap\SetPoint BL, idTweaksExperienceBar, BR, bar_margin, bar_margin
    else
      _, height = ActionBars\getTotalSize!
      size = height
      Minimap\SetPoint BL, idTweaksActionBar1, BR, bar_margin, bar_margin

    Minimap\SetWidth size
    Minimap\SetHeight size

  shape_position_disable: =>
    Minimap\ClearAllPoints!
    Minimap\SetParent MinimapCluster
    Minimap\SetPoint MC, MinimapCluster, TC, 9, -92
    Minimap\SetWidth 140
    Minimap\SetHeight 140

  mouse_check: =>
    @mouse_mousewheel_zooming_check!
     -- @mouse:checkRightClickMenuEnabled!

  mouse_disable: =>
    @mouse_mousewheel_zooming_disable!
     -- @mouse:disableRightClickMenu!

  mouse_mousewheel_zooming_check: =>
    if @db.mouse.mouse_wheel_zooming == true
      @mouse_mousewheel_zooming_enable!
    else
      @mouse_mousewheel_zooming_disable!

  mouse_mousewheel_zooming_enable: =>
    Minimap\EnableMouseWheel true
    Minimap\SetScript 'OnMouseWheel', (...) -> @mouse_handle_mousewheel_zoom ...

  mouse_mousewheel_zooming_disable: =>
    Minimap\EnableMouseWheel false
    Minimap\SetScript 'OnMouseWheel', nil

  mouse_handle_mousewheel_zoom: (frame, delta) =>
    if delta > 0 and Minimap\GetZoom! < 5
      Minimap\SetZoom Minimap\GetZoom! + 1
    elseif delta < 0 and Minimap\GetZoom! > 0
      Minimap\SetZoom Minimap\GetZoom! - 1

minimap_module = MiniMap idTweaks

GetMinimapShape = => minimap_module.db.profile.shape.square == true and 'SQUARE' or 'ROUND'

idTweaks\add_module minimap_module

-- function module.mouse:checkRightClickMenuEnabled()
--   if module.db.profile.mouse.right_click_menu == true then
--     module.mouse:enableRightClickMenu()
--   else
--     module.mouse:disableRightClickMenu()
--   end
-- end
--
-- function module.mouse:enableRightClickMenu()
--   module.dropdownItems = {
--     A = 'One',
--     B = 'Two',
--     C = 'Three',
--     D = 'Four',
--   }
--
--   local rightClickDropdown = AceGUI:Create('Dropdown')
--   rightClickDropdown:SetCallback('OnClose', function(frame)
--     module:Print('Dropdown:OnClose')
--     AceGUI:Release(frame)
--   end)
--   rightClickDropdown:SetText('AceGUI-3.0 Example')
--   rightClickDropdown:SetLabel('LABEL?')
--   rightClickDropdown:SetList(module.mouse.dropdownItems)
--   rightClickDropdown.frame:SetPoint(MC, Minimap, MC, 0, 0)
--
--   module.rightClickDropdown = rightClickDropdown
--
--   Minimap:SetScript('OnMouseUp', function(frame, button, down)
--     if button == 'RightButton' then
--       module.rightClickDropdown.frame:Show()
--     else
--       Minimap_OnClick(frame, button, down)
--     end
--   end)
-- end
--
-- function module.mouse:disableRightClickMenu()
--   Minimap:SetScript('OnMouseUp', nil)
-- end
